package com.diagbot.facade;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.SaveInquiryDTO;
import com.diagbot.entity.InquiryDetail;
import com.diagbot.entity.InquiryInfo;
import com.diagbot.entity.InquiryReport;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.InquiryDetailServiceImpl;
import com.diagbot.service.impl.InquiryInfoServiceImpl;
import com.diagbot.service.impl.InquiryReportServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.vo.SaveInquiryVO;

/**
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class InquiryInfoFacade extends InquiryInfoServiceImpl {
	
	@Autowired
    private InquiryDetailFacade inquiryDetailFacade;
	@Autowired
    private InquiryReportFacade inquiryReportFacade;
	@Autowired
    @Qualifier("inquiryDetailServiceImpl")
    private InquiryDetailServiceImpl inquiryDetailServiceImpl;
	@Autowired
    @Qualifier("inquiryReportServiceImpl")
    private InquiryReportServiceImpl inquiryReportServiceImpl;

    
	public SaveInquiryDTO saveInquiry(SaveInquiryVO saveInquiryVO){
		SaveInquiryDTO saveInquiryDTO = new SaveInquiryDTO();
        Date now = DateUtil.now();
        
        QueryWrapper<InquiryInfo> inquiryInfoQe = new QueryWrapper<>();
        inquiryInfoQe.eq("is_deleted", IsDeleteEnum.N.getKey());
        inquiryInfoQe.eq("hospital_id", saveInquiryVO.getHospitalId());
        inquiryInfoQe.eq("hospital_dept_id", saveInquiryVO.getHospitalDeptId());
        inquiryInfoQe.eq("doctor_id", saveInquiryVO.getDoctorId());
        inquiryInfoQe.eq("patient_id", saveInquiryVO.getPatientId());
        inquiryInfoQe.eq("inquiry_code", saveInquiryVO.getInquiryCode());
        InquiryInfo inquiryInfo = this.getOne(inquiryInfoQe);
        
        if(inquiryInfo==null){
        	inquiryInfo = new InquiryInfo();
        	inquiryInfo.setGmtCreate(now);
        }else{
        	QueryWrapper<InquiryDetail> inquiryDetailQe = new QueryWrapper<>();
        	inquiryDetailQe.eq("inquiry_id", inquiryInfo.getId());
        	inquiryDetailFacade.remove(inquiryDetailQe);
        	
        	QueryWrapper<InquiryReport> inquiryReportQe = new QueryWrapper<>();
        	inquiryReportQe.eq("inquiry_id", inquiryInfo.getId());
        	inquiryReportFacade.remove(inquiryReportQe);
        }
        
        BeanUtil.copyProperties(saveInquiryVO, inquiryInfo);
        inquiryInfo.setGmtModified(now);
        this.saveOrUpdate(inquiryInfo);
        
        Long inquiryId = inquiryInfo.getId();
        
        List<InquiryDetail> inquiryDetailList = BeanUtil.listCopyTo(saveInquiryVO.getDetailList(), InquiryDetail.class);
        inquiryDetailList.forEach(i->{
        	i.setInquiryId(inquiryId);
            i.setGmtCreate(now);
            i.setGmtModified(now);
        });
        inquiryDetailServiceImpl.saveBatch(inquiryDetailList);
        
        List<InquiryReport> inquiryReportList = BeanUtil.listCopyTo(saveInquiryVO.getReportList(), InquiryReport.class);
        inquiryReportList.forEach(i->{
        	i.setInquiryId(inquiryId);
            i.setGmtCreate(now);
            i.setGmtModified(now);
        });
        inquiryReportServiceImpl.saveBatch(inquiryReportList);
		
        saveInquiryDTO.setInquiryId(inquiryId);
        
		return saveInquiryDTO;
	}

}
